package com.wd.barcode_scanner_mlkit.analyzer

import android.annotation.SuppressLint
import android.graphics.Rect
import android.graphics.RectF
import android.os.Build
import androidx.annotation.RequiresApi
import androidx.camera.core.ImageAnalysis
import androidx.camera.core.ImageProxy
import com.google.mlkit.vision.barcode.Barcode
import com.google.mlkit.vision.barcode.BarcodeScannerOptions
import com.google.mlkit.vision.barcode.BarcodeScanning
import com.google.mlkit.vision.common.InputImage
import com.wd.barcode_scanner_mlkit.barcode.BarcodeBoxView

class ImageAnalyzer(
    private val barcodeBoxView: BarcodeBoxView,
    private val previewViewWidth: Float,
    private val previewViewHeight: Float,
    private val onBarcodeResult: (List<String>) -> Unit
) : ImageAnalysis.Analyzer {
    private val options by lazy {
        BarcodeScannerOptions.Builder()
            .setBarcodeFormats(Barcode.FORMAT_CODABAR)
            .build()
    }

    private var scaleX = 1f
    private var scaleY = 1f

    private fun translateX(x: Float) = x * scaleX
    private fun translateY(y: Float) = y * scaleY

    private fun adjustBoundingRect(rect: Rect) = RectF(
        translateX(rect.left.toFloat()),
        translateY(rect.top.toFloat()),
        translateX(rect.right.toFloat()),
        translateY(rect.bottom.toFloat())
    )

    private val scanner by lazy { BarcodeScanning.getClient(options) }

    @RequiresApi(Build.VERSION_CODES.KITKAT)
    @SuppressLint("UnsafeExperimentalUsageError", "UnsafeOptInUsageError")
    override fun analyze(image: ImageProxy) {
        scaleX = previewViewWidth / image.height.toFloat()
        scaleY = previewViewHeight / image.width.toFloat()
        val inputImage = image.image?.let { InputImage.fromMediaImage(it, 0) } ?: return
        scanner.process(inputImage)
            .addOnSuccessListener { barcodes ->
                barcodes
                    .mapNotNull {
                        it.boundingBox?.let { rect ->
                            barcodeBoxView.setRect(
                                adjustBoundingRect(
                                    rect
                                )
                            )
                        }
                        it.rawValue
                    }
                    .apply(onBarcodeResult)
            }
            .addOnFailureListener { }
        image.close()
    }

}